from .buff import Buff

HUNGRY = Buff("Hungry", "assets/buffs/hungry.png")
THIRSTY = Buff("Thirsty", "assets/buffs/thirsty.png")
POD_XP = Buff("XP Buff", "assets/buffs/pod_buff.png")
BROKEN_BONES = Buff("Broken Bones", "assets/buffs/broken_bones.png")