INVENTORY_OPEN_INTERVAL: int | float = 5
INVENTORY_CLOSE_INTERVAL: int | float = 5
TIMER_FACTOR: int | float = 1
ARK_PATH: str = "F:\ARKSurvivalEvolved"
TESSERACT_PATH: str = r"C:\Program Files\Tesseract-OCR\tesseract.exe"