from .item import Item

RIOT_HELMET = Item(
    name="Riot Helmet",
    search_name="helmet",
    stack_size=1,
    inventory_icon="assets/items/armor/riot_helmet.png",
)

RIOT_CHEST = Item(
    name="Riot Chest",
    search_name="chest",
    stack_size=1,
    inventory_icon="assets/items/armor/riot_chest.png",
)

RIOT_LEGGS = Item(
    name="Riot Leggings",
    search_name="legg",
    stack_size=1,
    inventory_icon="assets/items/armor/riot_leggs.png",
)

RIOT_GAUNTLETS = Item(
    name="Riot Gauntlets",
    search_name="gaunt",
    stack_size=1,
    inventory_icon="assets/items/armor/riot_gauntlets.png",
)

RIOT_BOOTS = Item(
    name="Riot Boots",
    search_name="boots",
    stack_size=1,
    inventory_icon="assets/items/armor/riot_boots.png",
)

MINER_HELMET = Item(
    name="Miner Helmet",
    search_name="miner helmet",
    stack_size=1,
    inventory_icon="assets/items/armor/miner_helmet.png",
)

RIOT = Item(
    name="Riot",
    search_name="riot",
    stack_size=1,
    inventory_icon="assets/items/armor/riot_chest.png",
)

TEK_GAUNTLETS = Item(
    name="Tek Gauntlets",
    search_name="tek gaunt",
    stack_size=1,
    inventory_icon="assets/items/armor/tek_gauntlets.png",
)

EXO_GLOVES = Item(
    name="Exo Gloves",
    search_name="exo gloves",
    stack_size=1,
    inventory_icon="assets/items/armor/exo_gauntlets.png",
)