from .item import Item

MEJOBERRY = Item(
    name="Mejoberry",
    search_name="mejoberry",
    stack_size=100,
    inventory_icon="assets/items/consumables/mejoberries.png",
)

NARCOBERRY = Item(
    name="Narcoberry",
    search_name="narcoberry",
    stack_size=100,
    inventory_icon="assets/items/consumables/narcoberries.png",
)

STIMBERRY = Item(
    name="Stimberry",
    search_name="stimberry",
    stack_size=100,
    inventory_icon="assets/items/consumables/stimberries.png",
)

TINTOBERRY = Item(
    name="Tintoberry",
    search_name="tintoberry",
    stack_size=100,
    inventory_icon="assets/items/consumables/tintoberries.png",
)

AMARBERRY = Item(
    name="Amarberry",
    search_name="amarberry",
    stack_size=100,
    inventory_icon="assets/items/consumables/amarberries.png",
)

AZULBERRY = Item(
    name="Azulberry",
    search_name="azulberry",
    stack_size=100,
    inventory_icon="assets/items/consumables/azulberries.png",
)

RAW_MEAT = Item(
    name="Raw Meat",
    search_name="meat",
    stack_size=40,
    inventory_icon="assets/items/consumables/raw_meat.png",
)

RAW_FISH_MEAT = Item(
    name="Raw Fish Meat",
    search_name="fish",
    stack_size=40,
    inventory_icon="assets/items/consumables/raw_fish.png",
)

SPOILED_MEAT = Item(
    name="Spoiled Meat",
    search_name="spoiled",
    stack_size=100,
    inventory_icon="assets/items/consumables/spoiled_meat.png",
)

PELLET = Item(
    name="Snow Owl Pellet",
    search_name="pellet",
    stack_size=1,
    inventory_icon="assets/items/consumables/pellet.png",
)


YTRAP_SEED = Item(
    name="Plant Species Y Seed",
    search_name="y seed",
    stack_size=100,
    inventory_icon="assets/items/consumables/ytrap_seed.png",
)